@@ -1,9 +1,12 @@
+from contextlib import suppress
 from pathlib import Path
 
-from pytoml import dump as write_toml
+from tomli_w import dump as write_toml
+from tomli_w import dumps as encode_toml
 
 from ..Helpers.structures import File, Intermediate, Resource, Writer
 from ..Helpers.utils import get_hash, get_name_from_scheme
+from .. import config
 
 
 class packwiz(Writer):
@@ -26,63 +29,81 @@ class packwiz(Writer):
         data = {
             "name": resource.name,
             "filename": resource.file.name,
+            "side": "both",
 
+            "download": {},
             "update": {}
         }
 
         slug = None
 
-        if provider := resource.providers.get("CurseForge"):
+        for prior in config.providers_priority:  
 
-            slug = provider.slug
-            
-            data['update']['curseforge'] = {
-                "file-id": provider.fileID,
-                "project-id": provider.ID,
-                "release-channel": "beta"
-            }
+            if prior == "CurseForge" and (provider := resource.providers.get("CurseForge")):
 
-            data['download'] = {
-                "hash-format": "sha1",
-                "hash": resource.file.hash.sha1,
-                "mode": "metadata:curseforge"
-            }
+                slug = provider.slug
+                
+                data['update']['curseforge'] = {
+                    "file-id": provider.fileID,
+                    "project-id": provider.ID,
+                    "release-channel": "beta"
+                }
 
-        elif provider := resource.providers.get("Modrinth"):
+                data['download'] = {
+                    "hash-format": "sha1",
+                    "hash": resource.file.hash.sha1,
+                    "mode": "metadata:curseforge"
+                }
 
-            slug = provider.slug
+                break
 
-            data['update']['modrinth'] = {
-                "mod-id": provider.ID,
-                "version": provider.fileID
-            }
+            if  prior == "Modrinth" and (provider := resource.providers.get("Modrinth")):
 
-            data['download'] = {
-                "url": provider.url,
-                "hash-format": "sha512",
-                "hash": resource.file.hash.sha512
-            }
+                slug = provider.slug
 
-        elif provider := resource.providers.get("Other"):
+                data['update']['modrinth'] = {
+                    "mod-id": provider.ID,
+                    "version": provider.fileID
+                }
 
-            slug = provider.slug
+                data['download'] = {
+                    "url": provider.url,
+                    "hash-format": "sha512",
+                    "hash": resource.file.hash.sha512
+                }
 
-            data['download'] = {
-                "url": provider.url,
-                "hash-format": "sha256",
-                "hash": resource.file.hash.sha256
-            }
+                break
+
+            if prior == "Other" and (provider := resource.providers.get("Other")):
+
+                slug = provider.slug
+
+                data['download'] = {
+                    "url": provider.url,
+                    "hash-format": "sha256",
+                    "hash": resource.file.hash.sha256
+                }
+
+                break
 
         if resource.optional: data['option'] = {"optional": True}
 
         from werkzeug.utils import secure_filename
         if not slug: slug = secure_filename(resource.name)
 
-        toml_path = self.temp_dir / resource.file.relativePath / (slug + ".toml")
+        toml_path = self.temp_dir / resource.file.relativePath / (slug + ".pw.toml")
         toml_path.parent.mkdir(parents=True, exist_ok=True)
 
         with open(toml_path, "w") as file:
-            write_toml(data, file)
+
+            if not data['update']: del data['update']
+            toml_data = encode_toml(data)
+
+            with suppress(ValueError):
+                index = toml_data.index("[update")
+                toml_data = toml_data[:index] + "[update]\n" + toml_data[index:]
+
+            file.write(toml_data)
 
         index_data = {
             "file": toml_path.relative_to(self.temp_dir).as_posix(),
@@ -101,7 +122,7 @@ class packwiz(Writer):
         copy_file(file.path, file_path)
 
         data = {
-            "file": Path(file.relativePath) / file.name,
+            "file": Path(file.relativePath).joinpath(file.name).as_posix(),
             "hash": file.hash.sha256
         }
         
@@ -116,13 +137,14 @@ class packwiz(Writer):
             self.add_resource(resource)
 
         index_path = self.temp_dir / "index.toml"
-        with open(index_path, "w") as file:
+        with open(index_path, "wb") as file:
             write_toml(self.index, file)
 
         self.pack_info = {
             "name": self.intermediate.name,
             "author": self.intermediate.author,
             "version": self.intermediate.version,
+            "pack-format": "packwiz:1.1.0",
 
             "index": {
                 "file": "index.toml",
@@ -136,7 +158,7 @@ class packwiz(Writer):
             }
         }
 
-        with open(self.temp_dir / "pack.toml", "w") as file:
+        with open(self.temp_dir / "pack.toml", "wb") as file:
             write_toml(self.pack_info, file)
 
         from shutil import make_archive
